// Copyright 2006 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.dom;

/**
 * Used by a the DOM to determine how to produce markup. Delegates details about converted entities
 * and some formatting details.
 */
public interface MarkupModel
{
    /**
     * Encodes the characters into the buffer, converting control characters (such as '&lt;') into
     * corresponding entities (such as &amp;lt;).
     * 
     * @param content
     *            to be filtered
     * @param buffer
     *            to recieve the filtered content
     */
    void encode(String content, StringBuilder buffer);

    /**
     * For a given element, determines how the end tag for the element should be rendered.
     */
    EndTagStyle getEndTagStyle(String element);
}
